package main

import (
	"database/sql"
	"net"

	migration "bitbucket.org/prmunty/util/dbmigration/db"
	"bitbucket.org/prmunty/util/ethernet"
	_ "github.com/mattn/go-sqlite3"
)

func main() {
	const version float64 = 8.0

	migration.Migrate(version, func(tx *sql.Tx) (err error) {
		return migration.DoUntilErr(tx,
			UpdateEquipmentTable,
			CreateModeOrdinalsTable,
			InsertModeOrdinals,
			UpdateFreezeTable,
			InsertFreezeData,
		)
	})

}

func UpdateEquipmentTable(tx *sql.Tx) error {
	rows, err := tx.Query(
		`SELECT ip FROM equipment`,
	);

	if err != nil {
		return err
	}

	for rows.Next() {
		var ip string

		err = rows.Scan(&ip)
		if err != nil {
			return err
		}

		netIp := net.ParseIP(ip)

		if ethernet.IsLocalIP(netIp) {
			if _, err = tx.Exec(
				`DELETE FROM equipment WHERE ip = ?`,
				ip,
			); err != nil {
				return err
			}
		}
	}

	return nil
}

func CreateModeOrdinalsTable(tx *sql.Tx) error {
	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS mode_ordinals; CREATE TABLE mode_ordinals(mode_data Integer,text_index Integer,ordinal Integer)`,
	); err != nil {
		return err
	}

	return nil
}

func InsertModeOrdinals(tx *sql.Tx) error {
	userdefinedIdsCond := map[int]int{
		69:2,
		70:1,
		71:3,
		72:4,
		73:5,
		90:9,
		91:10,
		92:11,
		93:12,
		94:13,
		95:14,
		96:15,
		97:16,
		98:17,
		99:18,
		170:6,
		171:7,
		172:8,
	}

	userdefinedIdsVess := map[int]int{
		0:1,
		1:11,
		2:21,
		3:2,
		4:12,
		5:22,
		21:3,
		22:13,
		23:23,
		24:4,
		25:14,
		26:24,
		27:5,
		28:15,
		29:25,
		30:6,
		31:16,
		32:26,
		33:7,
		34:17,
		35:27,
		36:8,
		37:18,
		38:28,
		39:9,
		40:19,
		41:29,
		42:10,
		43:20,
		44:30,
		54:36,
		55:36,
		56:37,
		57:38,
		58:39,
		59:40,
		60:41,
		61:42,
		62:43,
		63:44,
		110:31,
		111:32,
		160:33,
		161:34,
		162:35,
	}

	for key, value := range userdefinedIdsCond {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			669,
			key,
			value,
		); err != nil {
			return err
		}
	}

	for key, value := range userdefinedIdsCond {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			601,
			key,
			value,
		); err != nil {
			return err
		}
	}

	for key, value := range userdefinedIdsVess {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			611,
			key,
			value,
		); err != nil {
			return err
		}
	}

	for key, value := range userdefinedIdsVess {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			670,
			key,
			value,
		); err != nil {
			return err
		}
	}

	for key, value := range userdefinedIdsVess {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			860,
			key,
			value,
		); err != nil {
			return err
		}
	}

	return nil
}

func UpdateFreezeTable(tx *sql.Tx) error {
	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS "freeze_data_condenser"; CREATE TABLE "freeze_data_condenser" ("freeze_id" Integer PRIMARY KEY, "table_id" Integer, "selected" Boolean,"ordinal" Integer);`,
	); err != nil {
		return err
	}

	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS "freeze_data_vessel"; CREATE TABLE "freeze_data_vessel" ("freeze_id" Integer PRIMARY KEY, "table_id" Integer, "section" Integer, "selected" Boolean,"ordinal" Integer);`,
	); err != nil {
		return err
	}

	return nil
}


func InsertFreezeData(tx *sql.Tx) error {
	freezeIdsCond := map[int]int {
		134:9,
		135:10,
		136:11,
		137:12,
		138:13,
		139:14,
		140:15,
		141:16,
		142:17,
		143:18,
		1560:3,
		2021:4,
		2343:6,
		2344:7,
		2345:8,
		2358:5,
		2407:1,
		2408:2,
		2469:19,
		2470:20,
		2471:21,
		2472:22,
		2473:23,
		2474:24,
		2475:25,
		2476:26,
		2534:27,
		2535:28,
		2536:29,
		2537:30,
  }

	freezeIdsVess := map[int]int {
		959:1,
		971:11,
		983:21,
		960:2,
		972:12,
		984:22,
		962:3,
		974:13,
		986:23,
		964:4,
		976:14,
		988:24,
		966:5,
		978:15,
		990:25,
		968:6,
		980:16,
		992:26,
		961:7,
		973:17,
		985:27,
		963:8,
		975:18,
		987:28,
		965:9,
		977:19,
		989:29,
		967:10,
		979:20,
		991:30,
		586:36,
		587:37,
		588:38,
		589:39,
		590:40,
		591:41,
		592:42,
		593:43,
		594:44,
		595:45,
		282:31,
		283:32,
		2444:33,
		2445:34,
		2446:35,
		969:46,
		981:47,
		993:48,
		970:49,
		982:50,
		994:51,
		2534:52,
		2535:53,
		2536:54,
		2537:55,
	}

	freezeIdsSourceVess := map[int]int{
		959:1,
		971:2,
		983:3,
		960:1,
		972:2,
		984:3,
		962:1,
		974:2,
		986:3,
		964:1,
		976:2,
		988:3,
		966:1,
		978:2,
		990:3,
		968:1,
		980:2,
		992:3,
		961:1,
		973:2,
		985:3,
		963:1,
		975:2,
		987:3,
		965:1,
		977:2,
		989:3,
		967:1,
		979:2,
		991:3,
		586:4,
		587:4,
		588:4,
		589:4,
		590:4,
		591:4,
		592:4,
		593:4,
		594:4,
		595:4,
		282:4,
		283:4,
		2444:4,
		2445:4,
		2446:4,
		969:1,
		981:2,
		993:3,
		970:1,
		982:2,
		994:3,
		2534:4,
		2535:4,
		2536:4,
		2537:4,
	}

	for key, value := range freezeIdsCond {
		table_id := 0

		if _, err := tx.Exec(
			"INSERT INTO \"freeze_data_condenser\" (\"freeze_id\",\"table_id\",\"selected\",\"ordinal\") VALUES(?, ?, ?, ?);",
			key,
			table_id,
			1,
			value,
		); err != nil {
			return err
		}
	}

	for key, value := range freezeIdsVess {
		table_id := 0
		source_id := freezeIdsSourceVess[key]

		if _, err := tx.Exec(
			"INSERT INTO \"freeze_data_vessel\" (\"freeze_id\",\"table_id\",\"section\",\"selected\",\"ordinal\") VALUES(?, ?, ?, ?, ?);",
			key,
			table_id,
			source_id,
			1,
			value,
		); err != nil {
			return err
		}
	}

	return nil
}
